from enum import Enum
from pih.consts.polibase import POLIBASE
from pih.collections import (
    StorageVariableHolder,
    IntStorageVariableHolder,
    BoolStorageVariableHolder,
    TimeStorageVariableHolder,
    FloatStorageVariableHolder,
    IntListStorageVariableHolder,
    DateListStorageVariableHolder,
    StringListStorageVariableHolder,
)


class SETTINGS(Enum):
    REVIEW_ACTION_URL = StorageVariableHolder(
        "REVIEW_ACTION_URL",
        POLIBASE.REVIEW_ACTION_URL,
    )

    REVIEW_ACTION_URL_FOR_INPATIENT = StorageVariableHolder(
        "REVIEW_ACTION_URL_FOR_INPATIENT",
        POLIBASE.REVIEW_ACTION_URL_FOR_INPATIENT,
    )

    USER_RESPONSIBLE_FOR_PATIENT_MARK = StorageVariableHolder(
        "USER_RESPONSIBLE_FOR_PATIENT_MARK",
        "baa",
    )

    WIFI_VIP_PASSWORD = StorageVariableHolder("WIFI_VIP_PASSWORD", "ilovepacific")

    PLAIN_FORMAT_AS_DEFAULT_LOGIN_LIST = StringListStorageVariableHolder(
        "PLAIN_FORMAT_AS_DEFAULT_LOGIN_LIST", ("bar", "rob", "ptyu")
    )

    CHILLER_RECOGNIZE_LOG_LEVEL = IntStorageVariableHolder(
        "CHILLER_RECOGNIZE_LOG_LEVEL", 0
    )

    HEART_BEAT_IS_ON = BoolStorageVariableHolder("HEART_BEAT_IS_ON", True)

    CT_INDICATIONS_VALUE_TEMPERATURE_CORRECTION = FloatStorageVariableHolder(
        "CT_INDICATIONS_VALUE_TEMPERATURE_CORRECTION", 0.7
    )

    CT_INDICATIONS_VALUE_SAVE_PERIOD_IN_MINUTES = IntStorageVariableHolder(
        "CT_INDICATIONS_VALUE_SAVE_PERIOD_IN_MINUTES", 60
    )

    CHILLER_INDICATIONS_VALUE_SAVE_PERIOD_IN_MINUTES = IntStorageVariableHolder(
        "CHILLER_INDICATIONS_VALUE_SAVE_PERIOD_IN_MINUTES", 60
    )

    HOSPITAL_WORK_DAY_START_TIME = TimeStorageVariableHolder(
        "HOSPITAL_WORK_DAY_START_TIME", "8:00"
    )
    HOSPITAL_WORK_DAY_END_TIME = TimeStorageVariableHolder(
        "HOSPITAL_WORK_DAY_END_TIME", "20:00"
    )
    OFFICE_WORK_DAY_START_TIME = TimeStorageVariableHolder(
        "OFFICE_WORK_DAY_START_TIME", "8:30"
    )
    OFFICE_WORK_DAY_END_TIME = TimeStorageVariableHolder(
        "OFFICE_WORK_DAY_END_TIME", "17:00"
    )

    INDICATION_CT_NOTIFICATION_START_TIME = DateListStorageVariableHolder(
        "INDICATION_CT_NOTIFICATION_START_TIME", ("8:00", "12:00", "15:00", "17:00")
    )

    USER_USE_CACHE = BoolStorageVariableHolder("USER_USE_CACHE", True)

    POLIBASE_PERSON_INFORMATION_QUEST_IS_ON = BoolStorageVariableHolder(
        "POLIBASE_PERSON_INFORMATION_QUEST_IS_ON", False
    )
    #
    POLIBASE_PERSON_REVIEW_NOTIFICATION_IS_ON = BoolStorageVariableHolder(
        "POLIBASE_PERSON_REVIEW_NOTIFICATION_IS_ON", True
    )

    POLIBASE_PERSON_REVIEW_NOTIFICATION_ASK_WITHOUT_CHECK_FOR_CONFIRMATION = (
        BoolStorageVariableHolder(
            "POLIBASE_PERSON_REVIEW_NOTIFICATION_ASK_WITHOUT_CHECK_FOR_CONFIRMATION",
            True,
        )
    )

    POLIBASE_PERSON_REVIEW_NOTIFICATION_DAY_DELTA = IntStorageVariableHolder(
        "POLIBASE_PERSON_REVIEW_NOTIFICATION_DAY_DELTA", 0
    )

    POLIBASE_PERSON_REVIEW_NOTIFICATION_TEXT_FOR_CONFIRMED_NOTIFICATION = (
        StorageVariableHolder(
            "POLIBASE_PERSON_REVIEW_NOTIFICATION_TEXT_FOR_CONFIRMED_NOTIFICATION",
            POLIBASE.PERSON_REVIEW_NOTIFICATION_TEXT_FOR_CONFIRMED_NOTIFICATION,
        )
    )

    POLIBASE_PERSON_REVIEW_NOTIFICATION_TEXT = StorageVariableHolder(
        "POLIBASE_PERSON_REVIEW_NOTIFICATION_TEXT",
        POLIBASE.PERSON_REVIEW_NOTIFICATION_TEXT,
    )

    TIME_TRACKING_FOR_POLYCLINIC = StringListStorageVariableHolder(
        "TIME_TRACKING_FOR_POLYCLINIC",
        (
            "190",
            "035",
            "058",
            "101-0-",
            "125",
            "131",
            "139",
            "177",
            "124",
            "183",
            "045"
        ),
    )

    POLIBASE_PERSON_REVIEW_NOTIFICATION_START_TIME = TimeStorageVariableHolder(
        "POLIBASE_PERSON_REVIEW_NOTIFICATION_START_TIME", "23:00"
    )
    #
    RESOURCE_MANAGER_CHECK_SITE_CERTIFICATE_START_TIME = TimeStorageVariableHolder(
        "RESOURCE_MANAGER_CHECK_SITE_CERTIFICATE_START_TIME", "8:00"
    )
    #
    POLIBASE_CREATION_DB_DUMP_START_TIME = TimeStorageVariableHolder(
        "POLIBASE_CREATION_DB_DUMP_START_TIME", "20:30"
    )
    #
    RESOURCE_MANAGER_CHECK_SITE_FREE_SPACE_PERIOD_IN_MINUTES = IntStorageVariableHolder(
        "RESOURCE_MANAGER_CHECK_SITE_FREE_SPACE_PERIOD_IN_MINUTES", 15
    )
    #
    PRINTER_REPORT_PERIOD_IN_MINUTES = IntStorageVariableHolder(
        "PRINTER_REPORT_PERIOD_IN_MINUTES", 5
    )

    PRINTER_MINIMAL_REPORT_VALUE = IntStorageVariableHolder(
        "PRINTER_MINIMAL_REPORT_VALUE", 10
    )

    OUTSORCE_PRINTER_REPORT_IS_ON = BoolStorageVariableHolder(
        "OUTSORCE_PRINTER_REPORT_IS_ON", True
    )

    OUTSORCE_PRINTER_REPORT_START_TIME = TimeStorageVariableHolder(
        "OUTSORCE_PRINTER_REPORT_TIME", "9:00"
    )
    #
    POLIBASE_PERSON_VISIT_GREETING_NOTIFICATION_TEXT_WITHOUT_DATE_FOR_CONFIRMED_NOTIFICATION = StorageVariableHolder(
        "POLIBASE_PERSON_VISIT_GREETING_NOTIFICATION_TEXT_WITHOUT_DATE_FOR_CONFIRMED_NOTIFICATION",
        POLIBASE.PERSON_VISIT_GREETING_NOTIFICATION_TEXT_FOR_CONFIRMED_NOTIFICATION
        + POLIBASE.SEND_TELEGRAM_BOT_TEXT,
    )

    POLIBASE_PERSON_VISIT_GREETING_NOTIFICATION_TEXT_WITHOUT_DATE = (
        StorageVariableHolder(
            "POLIBASE_PERSON_VISIT_GREETING_NOTIFICATION_TEXT_WITHOUT_DATE",
            POLIBASE.PERSON_VISIT_GREETING_NOTIFICATION_TEXT_WITHOUT_TEXT,
        )
    )

    POLIBASE_PERSON_VISIT_GREETING_NOTIFICATION_TEXT_FOR_CONFIRMED_NOTIFICATION = StorageVariableHolder(
        "POLIBASE_PERSON_VISIT_GREETING_NOTIFICATION_TEXT_FOR_CONFIRMED_NOTIFICATION",
        POLIBASE.PERSON_VISIT_GREETING_NOTIFICATION_TEXT_FOR_CONFIRMED_NOTIFICATION
        + POLIBASE.PERSON_VISIT_NOTIFICATION_WITH_TIME_TEXT
        + POLIBASE.SEND_TELEGRAM_BOT_TEXT
        + POLIBASE.HAVE_A_GOOD_DAY,
    )

    POLIBASE_PERSON_VISIT_GREETING_NOTIFICATION_TEXT = StorageVariableHolder(
        "POLIBASE_PERSON_VISIT_GREETING_NOTIFICATION_TEXT",
        POLIBASE.PERSON_VISIT_GREETING_NOTIFICATION_TEXT_BASE
        + POLIBASE.PERSON_VISIT_NOTIFICATION_WITH_TIME_TEXT
        + POLIBASE.ASK_TO_SEND_TELEGRAM_BOT_URL_TEXT
        + POLIBASE.HAVE_A_GOOD_DAY,
    )

    PERSON_VISIT_NOTIFICATION_HEADER = StorageVariableHolder(
        "PERSON_VISIT_NOTIFICATION_HEADER",
        POLIBASE.PERSON_VISIT_NOTIFICATION_HEADER,
    )

    POLIBASE_PERSON_VISIT_NOTIFICATION_TEXT = StorageVariableHolder(
        "POLIBASE_PERSON_VISIT_NOTIFICATION_TEXT",
        POLIBASE.PERSON_VISIT_NOTIFICATION_HEADER
        + POLIBASE.PERSON_VISIT_NOTIFICATION_APPOINTMENT_INFORMATION
        + POLIBASE.PERSON_VISIT_NOTIFICATION_WITH_TIME_TEXT
        + POLIBASE.HAVE_A_GOOD_DAY,
    )

    POLIBASE_PERSON_VISIT_REMINDER_TEXT = StorageVariableHolder(
        "POLIBASE_PERSON_VISIT_REMINDER_TEXT",
        POLIBASE.PERSON_VISIT_NOTIFICATION_HEADER
        + "*{name}*, напоминаем Вам о записи сегодня {visit_time}. Вы записаны на {appointment_information}."
        + POLIBASE.PERSON_VISIT_NOTIFICATION_TEXT_CANCEL_OR_REPLACE_RECEPTION
        + POLIBASE.HAVE_A_GOOD_DAY,
    )

    POLIBASE_PERSON_TAKE_TELEGRAM_BOT_URL_TEXT = StorageVariableHolder(
        "POLIBASE_PERSON_TAKE_TELEGRAM_BOT_URL_TEXT",
        POLIBASE.TAKE_TELEGRAM_BOT_URL_TEXT,
    )

    POLIBASE_PERSON_TAKE_TELEGRAM_BOT_URL_PERSONLESS_TEXT = StorageVariableHolder(
        "POLIBASE_PERSON_TAKE_TELEGRAM_BOT_URL_PERSONLESS_TEXT",
        POLIBASE.TAKE_TELEGRAM_BOT_URL_PERSONLESS_TEXT,
    )

    POLIBASE_PERSON_TAKE_REVIEW_ACTION_URL_TEXT = StorageVariableHolder(
        "POLIBASE_PERSON_TAKE_REVIEW_ACTION_URL_TEXT",
        POLIBASE.TAKE_REVIEW_ACTION_URL_TEXT,
    )

    POLIBASE_PERSON_REVIEW_NOTIFICATION_DOCTOR_PERSON_PIN_LIST = (
        IntListStorageVariableHolder(
            "POLIBASE_PERSON_REVIEW_NOTIFICATION_DOCTOR_PERSON_PIN_LIST",
            POLIBASE.POLIBASE_PERSON_REVIEW_NOTIFICATION_DOCTOR_PERSON_PIN_LIST,
            "Список врачей, участвующих в отзывах",
        )
    )

    BONUS_DOCTOR_PERSON_PIN_LIST = IntListStorageVariableHolder(
        "BONUS_DOCTOR_PERSON_PIN_LIST",
        POLIBASE.BONUS_DOCTOR_PERSON_PIN_LIST,
    )

    POLIBASE_DOCTEMPLATE_THRESHOLD = IntStorageVariableHolder(
        "POLIBASE_DOCTEMPLATE_THRESHOLD", 100
    )

    POLIBASE_PERSON_YES_ANSWER_VARIANTS = StringListStorageVariableHolder(
        "POLIBASE_PERSON_YES_ANSWER_VARIANTS", POLIBASE.YES_ANSWER
    )

    POLIBASE_PERSON_NO_ANSWER_VARIANTS = StringListStorageVariableHolder(
        "POLIBASE_PERSON_NO_ANSWER_VARIANTS", POLIBASE.NO_ANSWER
    )

    POLIBASE_PERSON_NO_ANSWER_ON_NOTIFICATION_CONFIRMATION_TEXT = StorageVariableHolder(
        "POLIBASE_PERSON_NO_ANSWER_ON_NOTIFICATION_CONFIRMATION_TEXT",
        "Хорошего дня",
    )

    POLIBASE_PERSON_REVIEW_QUEST_WAIT_TIME = IntStorageVariableHolder(
        "POLIBASE_PERSON_REVIEW_QUEST_WAIT_TIME", 15
    )

    POLIBASE_PERSON_VISIT_NEED_REGISTER_GREETING_NOTIFICATION = (
        BoolStorageVariableHolder(
            "POLIBASE_PERSON_VISIT_NEED_REGISTER_GREETING_NOTIFICATION", True
        )
    )

    POLIBASE_PERSON_VISIT_NEED_REGISTER_REMINDER_NOTIFICATION = (
        BoolStorageVariableHolder(
            "POLIBASE_PERSON_VISIT_NEED_REGISTER_REMINDER_NOTIFICATION", True
        )
    )

    POLIBASE_PERSON_VISIT_TIME_BEFORE_REMINDER_NOTIFICATION_IN_MINUTES = (
        IntStorageVariableHolder(
            "POLIBASE_PERSON_VISIT_TIME_BEFORE_REMINDER_NOTIFICATION_IN_MINUTES", 120
        )
    )

    POLIBASE_PERSON_VISIT_NOTIFICATION_TEST_TELEPHONE_NUMBER = StorageVariableHolder(
        "POLIBASE_PERSON_VISIT_NOTIFICATION_TEST_TELEPHONE_NUMBER", None
    )

    POLIBASE_PERSON_REVIEW_NOTIFICATION_TEST_TELEPHONE_NUMBER = StorageVariableHolder(
        "POLIBASE_PERSON_REVIEW_NOTIFICATION_TEST_TELEPHONE_NUMBER", None
    )

    WHATSAPP_SENDING_MESSAGES_VIA_WAPPI_IS_ON = BoolStorageVariableHolder(
        "WHATSAPP_SENDING_MESSAGES_VIA_WAPPI_IS_ON", True
    )

    WHATSAPP_BUFFERED_MESSAGE_MIN_DELAY_IN_MILLISECONDS = IntStorageVariableHolder(
        "WHATSAPP_BUFFERED_MESSAGE_MIN_DELAY_IN_MILLISECONDS", 6000
    )

    WHATSAPP_BUFFERED_MESSAGE_MAX_DELAY_IN_MILLISECONDS = IntStorageVariableHolder(
        "WHATSAPP_BUFFERED_MESSAGE_MAX_DELAY_IN_MILLISECONDS", 12000
    )

    WHATSAPP_MESSAGE_SENDER_USER_LOGIN = StorageVariableHolder(
        "WHATSAPP_MESSAGE_SENDER_USER_LOGIN", "Administrator"
    )

    POLIBASE_WAS_EMERGENCY_CLOSED_NOTIFICATION_TEXT = StorageVariableHolder(
        "POLIBASE_WAS_EMERGENCY_CLOSED_NOTIFICATION_TEXT",
        "к сожалениию наш Полибейс поломался и был аварийно закрыт, ожидайте сообщение о просьбе переоткрыть его!",
    )

    POLIBASE_WAS_RESTARTED_NOTIFICATION_TEXT = StorageVariableHolder(
        "POLIBASE_WAS_RESTARTED_NOTIFICATION_TEXT",
        "Полибейс перезагружен, можете переоткрыть его.",
    )

    WORKSTATION_SHUTDOWN_TIME = TimeStorageVariableHolder(
        "WORKSTATION_SHUTDOWN_TIME", "21:00"
    )

    WORKSTATION_REBOOT_TIME = TimeStorageVariableHolder(
        "WORKSTATION_REBOOT_TIME", "21:00"
    )

    EMAIL_VALIDATION_IS_ON = BoolStorageVariableHolder("EMAIL_VALIDATION_IS_ON", True)
    EMAIL_VALIDATION_TEST = BoolStorageVariableHolder("EMAIL_VALIDATION_TEST", False)

    CHILLER_ALERT_TEMPERATURE = FloatStorageVariableHolder(
        "CHILLER_ALERT_TEMPERATURE", 17.0
    )

    CHILLER_COUNT_DEFAULT = FloatStorageVariableHolder("CHILLER_COUNT_DEFAULT", 3)

    CHILLER_MAX_TEMPERATURE = IntStorageVariableHolder("CHILLER_MAX_TEMPERATURE", 16)

    CHILLER_MIN_TEMPERATURE = IntStorageVariableHolder("CHILLER_MIN_TEMPERATURE", 10)

    CHILLER_ACTION_MAX_TEMPERATURE = FloatStorageVariableHolder(
        "CHILLER_ACTION_MAX_TEMPERATURE", 15.1
    )

    CHILLER_ACTION_MIN_TEMPERATURE = IntStorageVariableHolder(
        "CHILLER_ACTION_MIN_TEMPERATURE", 11
    )

    CT_ROOM_MAX_TEMPERATURE = IntStorageVariableHolder("CT_ROOM_MAX_TEMPERATURE", 24)

    CT_ROOM_MIN_TEMPERATURE = IntStorageVariableHolder("CT_ROOM_MIN_TEMPERATURE", 20)

    CT_ROOM_MAX_HUMIDITY = IntStorageVariableHolder("CT_ROOM_MAX_HUMIDITY", 60)

    CT_ROOM_MIN_HUMIDITY = IntStorageVariableHolder("CT_ROOM_MIN_HUMIDITY", 30)

    CHECK_ALL_RECIPIENT_USER_LOGIN = StorageVariableHolder(
        "CHECK_ALL_RECIPIENT_USER_LOGIN", "nak"
    )

    MAIL_SITE_FREE_MEMORY_PERIOD = IntStorageVariableHolder(
        "MAIL_SITE_FREE_MEMORY_PERIOD", 180
    )

    SITE_FREE_MEMORY_PERIOD = IntStorageVariableHolder("SITE_FREE_MEMORY_PERIOD", 180)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rpcCommandCall.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14rpcCommandCall.proto\x12\x05unary\".\n\nrpcCommand\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\nparameters\x18\x02 \x01(\t\" \n\x10rpcCommandResult\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t2G\n\x05Unary\x12>\n\x0erpcCallCommand\x12\x11.unary.rpcCommand\x1a\x17.unary.rpcCommandResult\"\x00\x62\x06proto3')



_RPCCOMMAND = DESCRIPTOR.message_types_by_name['rpcCommand']
_RPCCOMMANDRESULT = DESCRIPTOR.message_types_by_name['rpcCommandResult']
rpcCommand = _reflection.GeneratedProtocolMessageType('rpcCommand', (_message.Message,), {
  'DESCRIPTOR' : _RPCCOMMAND,
  '__module__' : 'rpcCommandCall_pb2'
  # @@protoc_insertion_point(class_scope:unary.rpcCommand)
  })
_sym_db.RegisterMessage(rpcCommand)

rpcCommandResult = _reflection.GeneratedProtocolMessageType('rpcCommandResult', (_message.Message,), {
  'DESCRIPTOR' : _RPCCOMMANDRESULT,
  '__module__' : 'rpcCommandCall_pb2'
  # @@protoc_insertion_point(class_scope:unary.rpcCommandResult)
  })
_sym_db.RegisterMessage(rpcCommandResult)

_UNARY = DESCRIPTOR.services_by_name['Unary']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _RPCCOMMAND._serialized_start=31
  _RPCCOMMAND._serialized_end=77
  _RPCCOMMANDRESULT._serialized_start=79
  _RPCCOMMANDRESULT._serialized_end=111
  _UNARY._serialized_start=113
  _UNARY._serialized_end=184
# @@protoc_insertion_point(module_scope)



from pih.consts import CONST
from pih.consts.service_roles import ServiceRoles
from pih.tools import NetworkTool, OSTool, j, e, n
from pih.consts.service import EVENT_LISTENER_NAME_PREFIX, SUPPORT_NAME_PREFIX
from pih.collections.service import (
    ServiceDescriptionBase,
    ServiceInformation,
    ServiceDescription,
)


from enum import Enum


class ServiceTool:

    @staticmethod
    def is_service_as_listener(description: ServiceDescriptionBase) -> bool:
        return description.name.find(EVENT_LISTENER_NAME_PREFIX) == 0

    @staticmethod
    def is_service_as_support(description: ServiceDescriptionBase) -> bool:
        return description.name.find(SUPPORT_NAME_PREFIX) == 0


class ServiceAdminTool:

    service_collection: dict[ServiceDescriptionBase, ServiceInformation] = {}

    @staticmethod
    def remove_service_from_service_collection(value: ServiceDescriptionBase) -> None:
        if value in ServiceAdminTool.service_collection:
            del ServiceAdminTool.service_collection[value]

    @staticmethod
    def clear_service_collection() -> None:
        ServiceAdminTool.service_collection = {}

    @staticmethod
    def get_service_collection() -> dict[ServiceDescriptionBase, ServiceInformation]:
        return ServiceAdminTool.service_collection

    @staticmethod
    def update_service_information(
        value: ServiceDescriptionBase | list[ServiceDescriptionBase],
        add: bool = True,
        overwrite: bool = False,
    ) -> None:
        if not isinstance(value, list):
            value = [value]
        if overwrite:
            ServiceAdminTool.service_collection = {}
        for item in value:
            if add:
                ServiceAdminTool.service_collection[item] = item
            else:
                if item in ServiceAdminTool.service_collection:
                    del ServiceAdminTool.service_collection[item]

    @staticmethod
    def create_port(
        service_role_or_description: ServiceRoles | ServiceDescriptionBase,
    ) -> int:
        return (
            ServiceRoleTool.service_description(service_role_or_description).port
            or NetworkTool.next_free_port()
        )

    @staticmethod
    def create_host(
        service_role_or_description: ServiceRoles | ServiceDescriptionBase,
    ) -> str:
        description: ServiceDescription = ServiceRoleTool.service_description(
            service_role_or_description
        )
        return (
            OSTool.host()
            if description.isolated or e(description.host)
            else description.host
        )


class ServiceRoleTool:

    @staticmethod
    def service_description(
        value: Enum | str | ServiceDescriptionBase, get_source_description: bool = False
    ) -> ServiceDescriptionBase | None:
        def isolated_name(
            value: ServiceDescriptionBase | None,
        ) -> ServiceDescriptionBase | None:
            if n(value):
                return None
            value.name = (
                j((CONST.ISOLATED_ARG_NAME, value.name), CONST.SPLITTER)
                if value.isolated and value.name.find(CONST.ISOLATED_ARG_NAME) == -1
                else value.name
            )
            return value

        if isinstance(value, str):
            for service_role in ServiceRoles:
                if ServiceRoleTool.service_description(service_role).name == value:
                    return isolated_name(service_role.value)
            return None
        if isinstance(value, ServiceDescriptionBase):
            return isolated_name(
                ServiceRoleTool.service_description(value.name)
                if get_source_description
                else value
            )
        return isolated_name(value.value)



from enum import Enum
from pih.collections import EventDescription, ParamItem
from pih.consts import LogMessageChannels, LogMessageFlags
from pih.consts.names import PARAM_ITEMS, FIELD_NAME_COLLECTION


class Events(Enum):
    DEBUG = EventDescription(
        "It is a debug event", LogMessageChannels.DEBUG, LogMessageFlags.DEBUG
    )

    ERROR = EventDescription(
        "{}",
        LogMessageChannels.DEBUG,
        LogMessageFlags.ERROR,
        (
            PARAM_ITEMS.TEXT,
            PARAM_ITEMS.HOST.configurate(visible=False),
            PARAM_ITEMS.TYPE.configurate(visible=False),
            PARAM_ITEMS.ARGUMENTS.configurate(visible=False),
        ),
    )

    PRINTER_REPORT = EventDescription(
        "Принтер {printer_name} ({location}):\n {printer_report}",
        LogMessageChannels.PRINTER,
        LogMessageFlags.NORMAL,
        (
            ParamItem("printer_name", "Name of printer"),
            ParamItem("location", "Location"),
            ParamItem("printer_report", "Printer report"),
        ),
    )
    #
    ADD_JOURNAL_RECORD = EventDescription(
        "{tag_string}. {type_string}. {samAccountName} сделал запись в журнале. {title}: {text}.",
        LogMessageChannels.JOURNAL,
        (LogMessageFlags.NORMAL, LogMessageFlags.SAVE),
        (
            PARAM_ITEMS.TAG,
            PARAM_ITEMS.TYPE,
            PARAM_ITEMS.LOGIN,
            PARAM_ITEMS.TITLE,
            PARAM_ITEMS.TEXT,
            ParamItem("tag_string", "Tags string").configurate(saved=False),
            ParamItem("type_string", "Type string").configurate(saved=False),
            PARAM_ITEMS.PARAMETERS,
        ),
    )
    #
    LOG_IN = EventDescription(
        "Пользователь {} ({}) вошел с компьютера {}",
        LogMessageChannels.IT_BOT,
        LogMessageFlags.NORMAL,
        (
            PARAM_ITEMS.FULL_NAME,
            PARAM_ITEMS.LOGIN,
            ParamItem("computer_name", "Name of computer"),
        ),
    )

    SESSION_STARTED = EventDescription(
        "Пользователь {} ({}) начал пользоваться программой {}.\nВерсия: {}.\nНазвание компьютера: {}",
        LogMessageChannels.IT_BOT,
        LogMessageFlags.NORMAL,
        (
            PARAM_ITEMS.FULL_NAME,
            PARAM_ITEMS.LOGIN,
            ParamItem("app_name", "Name of user"),
            ParamItem("version", "Version"),
            ParamItem("computer_name", "Name of computer"),
        ),
    )

    SERVICE_WAS_STARTED = EventDescription(
        "Сервис {service_name} запущен!\nИмя хоста: {host_name}\nПорт: {port}\nИдентификатор процесса: {pid}\n",
        LogMessageChannels.SERVICES,
        LogMessageFlags.NORMAL,
        (
            PARAM_ITEMS.SERVICE_NAME,
            ParamItem("host_name", "Name of host"),
            ParamItem("port", "Port"),
            ParamItem("pid", "PID"),
            ParamItem("service_information", "Service information"),
        ),
    )

    SERVICE_IS_BEING_STARTED = EventDescription(
        "Сервис {service_name} запускается!\nИмя хоста: {host_name}\nПорт: {port}",
        LogMessageChannels.SERVICES,
        LogMessageFlags.NORMAL,
        (
            PARAM_ITEMS.SERVICE_NAME,
            ParamItem("host_name", "Name of host"),
            ParamItem("port", "Port"),
        ),
    )

    SERVICE_WAS_STOPPED = EventDescription(
        "Сервис {service_name} остановлен!",
        LogMessageChannels.SERVICES,
        LogMessageFlags.NORMAL,
        (
            PARAM_ITEMS.SERVICE_NAME,
            ParamItem("service_information", "Service information"),
        ),
    )

    SERVICE_WAS_NOT_STARTED = EventDescription(
        "Сервис {service_name} не запущен!\nИмя хоста: {host_name}\nПорт: {port}\nОшибка:{error}",
        LogMessageChannels.SERVICES,
        LogMessageFlags.ERROR,
        (
            PARAM_ITEMS.SERVICE_NAME,
            ParamItem("host_name", "Name of host"),
            ParamItem("port", "Port"),
            ParamItem("error", "Error"),
            ParamItem("service_information", "Service information"),
        ),
    )

    SERVICE_IS_INACCESIBLE_AND_WILL_BE_RESTARTED = EventDescription(
        "Сервис {service_name} недоступен и будет перезапущен!",
        LogMessageChannels.SERVICES,
        LogMessageFlags.ERROR,
        (
            PARAM_ITEMS.SERVICE_NAME,
            ParamItem("service_information", "Service  information"),
        ),
    )

    WHATSAPP_MESSAGE_RECEIVED = EventDescription(
        "Получено сообщение",
        LogMessageChannels.POLIBASE,
        LogMessageFlags.SILENCE,
        (ParamItem("message", "Сообщение"),),
    )

    NEW_FILE_DETECTED = EventDescription(
        "Новый файл: {path}",
        LogMessageChannels.IT_BOT,
        LogMessageFlags.NORMAL,
        (PARAM_ITEMS.PATH,),
    )

    NEW_POLIBASE_DOCUMENT_DETECTED = EventDescription(
        lambda args: [
            "Новый документ Полибейс: {path}",
            "Обработанный документ Полибейс: {path}",
        ][args[PARAM_ITEMS.STATUS.name]],
        LogMessageChannels.POLIBASE_DOCUMENT,
        (LogMessageFlags.NOTIFICATION, LogMessageFlags.SAVE_ONCE),
        (
            PARAM_ITEMS.PERSON_PIN,
            PARAM_ITEMS.PATH.configurate(key=True),
            PARAM_ITEMS.DOCUMENT_NAME,
            PARAM_ITEMS.STATUS,
        ),
    )

    COMPUTER_WAS_STARTED = EventDescription(
        "Компьютер {name} загрузился",
        LogMessageChannels.IT,
        LogMessageFlags.NORMAL,
        (ParamItem("name", "Название компьютера"),),
    )

    SERVER_WAS_STARTED = EventDescription(
        "Сервер {name} загрузился",
        LogMessageChannels.IT,
        LogMessageFlags.NORMAL,
        (ParamItem("name", "Название сервера"),),
    )

    RESOURCE_INACCESSABLE = EventDescription(
        "Ресурс {resource_name} недоступен. {reason_string}",
        LogMessageChannels.RESOURCES,
        LogMessageFlags.ERROR,
        (
            ParamItem("resource_name", "Название ресурса"),
            ParamItem("resource", "Ресурс"),
            ParamItem("at_first_time", "Признак первого раза"),
            ParamItem("reason_string", "Строка причины"),
            ParamItem("reason", "Причины", optional=True),
        ),
    )

    RESOURCE_ACCESSABLE = EventDescription(
        "Ресурс {resource_name} доступен",
        LogMessageChannels.RESOURCES,
        LogMessageFlags.NORMAL,
        (
            ParamItem("resource_name", "Название ресурса"),
            ParamItem("resource", "Ресурс"),
            ParamItem("at_first_time", "Признак первого раза"),
        ),
    )

    #
    BACKUP_ROBOCOPY_JOB_WAS_STARTED = EventDescription(
        "Robocopy: Начато выполнение задания: {name}. PID процесса: {pid}",
        LogMessageChannels.BACKUP,
        (LogMessageFlags.NOTIFICATION, LogMessageFlags.SAVE_ONCE),
        (PARAM_ITEMS.NAME.configurate(key=True), PARAM_ITEMS.PID),
    )

    BACKUP_ROBOCOPY_JOB_WAS_COMPLETED = EventDescription(
        "Robocopy: Завершено выполнение задания: {name}. Статус: {status_string}",
        LogMessageChannels.BACKUP,
        (LogMessageFlags.NOTIFICATION, LogMessageFlags.SAVE),
        (PARAM_ITEMS.NAME, ParamItem("status_string", ""), PARAM_ITEMS.STATUS),
    )
    #
    POLIBASE_CREATION_DB_DUMP_START = EventDescription(
        "Базы данных Polibase: Начато создание дампа",
        LogMessageChannels.BACKUP,
        (LogMessageFlags.NORMAL, LogMessageFlags.SAVE),
    )

    POLIBASE_CREATION_DB_DUMP_COMPLETE = EventDescription(
        "Базы данных Polibase: Завершено создание дампа. Размер: {size}",
        LogMessageChannels.BACKUP,
        (LogMessageFlags.NORMAL, LogMessageFlags.SAVE),
        (ParamItem("size", ""),),
    )

    POLIBASE_CREATION_ARCHIVED_DB_DUMP_START = EventDescription(
        "Базы данных Polibase: Начато архивирование дампа",
        LogMessageChannels.BACKUP,
        LogMessageFlags.NORMAL,
    )

    POLIBASE_CREATION_ARCHIVED_DB_DUMP_COMPLETE = EventDescription(
        "Базы данных Polibase: Завершено архивирование дампа. Размер: {size}",
        LogMessageChannels.BACKUP,
        (LogMessageFlags.NORMAL, LogMessageFlags.SAVE),
        (ParamItem("size", ""),),
    )

    POLIBASE_COPING_ARCHIVED_DB_DUMP_START = EventDescription(
        "Базы данных Polibase: Начато копирование архивированного дампа на {}",
        LogMessageChannels.BACKUP,
        LogMessageFlags.NORMAL,
        (PARAM_ITEMS.DESTINATION,),
    )

    POLIBASE_COPING_ARCHIVED_DB_DUMP_COMPLETE = EventDescription(
        "Базы данных Polibase: Завершено копирование архивированного дампа на {}",
        LogMessageChannels.BACKUP,
        LogMessageFlags.NORMAL,
        (PARAM_ITEMS.DESTINATION,),
    )

    POLIBASE_COPING_DB_DUMP_START = EventDescription(
        "Базы данных Polibase: Начато копирование дампа на {}",
        LogMessageChannels.BACKUP,
        LogMessageFlags.NORMAL,
        (PARAM_ITEMS.DESTINATION,),
    )

    POLIBASE_COPING_DB_DUMP_COMPLETE = EventDescription(
        "Базы данных Polibase: Завершено копирование дампа на {}",
        LogMessageChannels.BACKUP,
        LogMessageFlags.NORMAL,
        (PARAM_ITEMS.DESTINATION,),
    )
    #
    HR_NOTIFY_ABOUT_NEW_EMPLOYEE = EventDescription(
        "День добрый, {hr_given_name}.\nДокументы для нового сотрудника: {employee_full_name} готовы!\nЕго корпоративная почта: {employee_email}.",
        LogMessageChannels.HR,
        LogMessageFlags.NOTIFICATION.value,
        (
            ParamItem("hr_given_name", "Имя руководителя отдела HR"),
            ParamItem("employee_full_name", "ФИО нового сотрудника"),
            ParamItem("employee_email", "Корпаротивная почта нового сотрудника"),
        ),
    )
    #
    IT_NOTIFY_ABOUT_CREATE_USER = EventDescription(
        "Добрый день, отдел Информационных технологий.\nДокументы для нового пользователя: {} готовы!\nОписание: {}\nЛогин: {}\nПароль: {}\nТелефон: {}\nЭлектронная почта: {}\nДополнительная информация: {}",
        LogMessageChannels.IT,
        LogMessageFlags.NOTIFICATION.value,
        (
            PARAM_ITEMS.NAME,
            PARAM_ITEMS.DESCRIPTION,
            PARAM_ITEMS.LOGIN,
            PARAM_ITEMS.PASSWORD,
            PARAM_ITEMS.TELEPHONE_NUMBER,
            PARAM_ITEMS.EMAIL,
            ParamItem("information", ""),
        ),
    )

    IT_NOTIFY_ABOUT_CREATE_PERSON = EventDescription(
        "Добрый день, отдел Информационных технологий.\nДокументы для новой персоны: {name} готовы!\nОписание: {description}\nЭлектронная почта: {email}\nПароль: {password}\nТелефон: {telephoneNumber}",
        LogMessageChannels.IT,
        LogMessageFlags.NOTIFICATION.value,
        (
            PARAM_ITEMS.NAME,
            PARAM_ITEMS.DESCRIPTION,
            PARAM_ITEMS.EMAIL,
            PARAM_ITEMS.PASSWORD,
            PARAM_ITEMS.TELEPHONE_NUMBER,
        ),
    )

    IT_NOTIFY_ABOUT_CREATE_MARK = EventDescription(
        "Добрый день, отдел Информационных технологий.\nКарта доступа для новой персоны: {name} готова!\nТелефон: {telephoneNumber}\nНомер карты доступа: {TabNumber}\nГруппа доступа: {group_name}",
        LogMessageChannels.IT,
        LogMessageFlags.NOTIFICATION.value,
        (
            PARAM_ITEMS.NAME,
            PARAM_ITEMS.TELEPHONE_NUMBER,
            PARAM_ITEMS.TAB_NUMBER,
            ParamItem("group_name", ""),
        ),
    )

    IT_NOTIFY_ABOUT_CREATE_TEMPORARY_MARK = EventDescription(
        "Добрый день, отдел Информационных технологий.\nВременная карта доступа для персоны: {name} готова!\nНомер карты: {TabNumber}\nТелефон: {telephoneNumber}",
        LogMessageChannels.IT,
        LogMessageFlags.NOTIFICATION.value,
        (PARAM_ITEMS.NAME, PARAM_ITEMS.TAB_NUMBER, PARAM_ITEMS.TELEPHONE_NUMBER),
    )

    IT_NOTIFY_ABOUT_TEMPORARY_MARK_RETURN = EventDescription(
        "Добрый день, отдел Информационных технологий.\nВременная карта доступа для персоны: {name} возвращена!\nНомер карты: {TabNumber}",
        LogMessageChannels.IT,
        LogMessageFlags.NOTIFICATION.value,
        (PARAM_ITEMS.NAME, PARAM_ITEMS.TAB_NUMBER),
    )

    IT_NOTIFY_ABOUT_MARK_RETURN = EventDescription(
        "Добрый день, отдел Информационных технологий.\nКарта доступа для персоны: {name} возвращена!\nНомер карты: {TabNumber}",
        LogMessageChannels.IT,
        LogMessageFlags.NOTIFICATION.value,
        (PARAM_ITEMS.NAME, PARAM_ITEMS.TAB_NUMBER),
    )

    IT_TASK_AFTER_CREATE_USER = EventDescription(
        "Добрый день, {it_user_name}.\nНеобходимо создать ящик электронной почту для пользователя: {name}\nАдресс электронной почты: {mail}\nПароль: {password}",
        LogMessageChannels.IT,
        LogMessageFlags.TASK.value,
        (
            ParamItem("it_user_name", ""),
            PARAM_ITEMS.NAME,
            ParamItem("mail", ""),
            PARAM_ITEMS.PASSWORD,
        ),
    )

    IT_TASK_AFTER_CREATE_PERSON = EventDescription(
        "Добрый день, {it_user_name}.\nНеобходимо создать ящик электронной почты почту для персоны: {name}\nАдресс электронной почты: {mail}\nПароль: {password}",
        LogMessageChannels.IT,
        LogMessageFlags.TASK.value,
        (
            ParamItem("it_user_name", ""),
            PARAM_ITEMS.NAME,
            ParamItem("mail", ""),
            PARAM_ITEMS.PASSWORD,
        ),
    )

    WATCHABLE_WORKSTATION_IS_NOT_ACCESSABLE = EventDescription(
        "Компьютер {} вне сети",
        LogMessageChannels.RESOURCES,
        LogMessageFlags.ERROR,
        (PARAM_ITEMS.NAME,),
    )

    WATCHABLE_WORKSTATION_IS_ACCESSABLE = EventDescription(
        "Компьютер {} в сети",
        LogMessageChannels.RESOURCES,
        LogMessageFlags.NORMAL,
        (PARAM_ITEMS.NAME,),
    )

    SERVER_IS_NOT_ACCESSABLE = EventDescription(
        "Сервер {} вне сети",
        LogMessageChannels.RESOURCES,
        (LogMessageFlags.ERROR, LogMessageFlags.SAVE_ONCE),
        (PARAM_ITEMS.NAME,),
    )

    SERVER_IS_ACCESSABLE = EventDescription(
        "Сервер {} в сети",
        LogMessageChannels.RESOURCES,
        (LogMessageFlags.NORMAL, LogMessageFlags.SAVE_ONCE),
        (PARAM_ITEMS.NAME,),
    )

    # INDICATION_DEVICES

    INDICATION_DEVICES_WAS_REGISTERED = EventDescription(
        "Устройство индикации: {} ({}) зарегистрировано. Адресс: {}",
        LogMessageChannels.RESOURCES,
        (LogMessageFlags.NOTIFICATION, LogMessageFlags.SAVE_ONCE),
        (
            PARAM_ITEMS.NAME.configurate(key=True),
            PARAM_ITEMS.DESCRIPTION,
            PARAM_ITEMS.IP_ADDRESS,
        ),
    )

    # MRI

    MRI_CHILLER_FILTER_WAS_CHANGED = EventDescription(
        "Фильтр водяного охлаждения МРТ был заменён. Количество оставшихся фильтров: {}",
        LogMessageChannels.RESOURCES,
        (LogMessageFlags.NOTIFICATION, LogMessageFlags.SAVE),
        (PARAM_ITEMS.COUNT,),
    )

    MRI_CHILLER_TEMPERATURE_ALERT_WAS_FIRED = EventDescription(
        "Превышена температура чиллера",
        LogMessageChannels.RESOURCES,
        (LogMessageFlags.ERROR, LogMessageFlags.SAVE),
    )

    MRI_CHILLER_TEMPERATURE_ALERT_WAS_RESOLVED = EventDescription(
        "Проблема повышенной температуры чиллера была решена",
        LogMessageChannels.RESOURCES,
        (LogMessageFlags.NOTIFICATION, LogMessageFlags.SAVE),
    )

    MRI_CHILLER_WAS_TURNED_OFF = EventDescription(
        "Чиллер был выключен",
        LogMessageChannels.RESOURCES,
        (LogMessageFlags.NOTIFICATION, LogMessageFlags.SAVE),
    )

    MRI_CHILLER_WAS_TURNED_ON = EventDescription(
        "Чиллер был включен",
        LogMessageChannels.RESOURCES,
        (LogMessageFlags.NOTIFICATION, LogMessageFlags.SAVE),
    )

    # POLIBASE

    POLIBASE_PERSON_DUPLICATION_WAS_DETECTED = EventDescription(
        "Регистратор {registrator_person_name} создал персону {person_name} ({person_pin}), которая дублирует {duplicated_person_name} ({duplicated_person_pin})",
        LogMessageChannels.POLIBASE_ERROR,
        LogMessageFlags.SAVE,
        (
            ParamItem("person_name", ""),
            PARAM_ITEMS.PERSON_PIN,
            ParamItem("duplicated_person_name", ""),
            ParamItem("duplicated_person_pin", ""),
            ParamItem("registrator_person_name", ""),
        ),
    )

    POLIBASE_PERSON_VISIT_WAS_REGISTERED = EventDescription(
        "Зарегистрировано новое посещение: {name} ({type_string})",
        LogMessageChannels.POLIBASE,
        LogMessageFlags.NOTIFICATION,
        (PARAM_ITEMS.NAME, ParamItem("type_string", ""), ParamItem("visit", "")),
    )

    POLIBASE_PERSON_WAS_CREATED = EventDescription(
        "Создана полибейс персона: {name} ({pin})",
        LogMessageChannels.POLIBASE,
        LogMessageFlags.NOTIFICATION,
        (PARAM_ITEMS.NAME, PARAM_ITEMS.PIN, PARAM_ITEMS.VALUE),
    )

    POLIBASE_PERSON_ANSWERED = EventDescription(
        "Клиент: {} ({}) ответил: \"{}\" на получение ссылки",
        LogMessageChannels.POLIBASE,
        (LogMessageFlags.NOTIFICATION, LogMessageFlags.SAVE),
        (PARAM_ITEMS.PIN, PARAM_ITEMS.TELEPHONE_NUMBER, PARAM_ITEMS.VALUE),
    )

    MAIL_TO_POLIBASE_PERSON_WAS_SENT = EventDescription(
        "Почта с медицинской записью {id} была отправлена пациенту: {pin}",
        LogMessageChannels.POLIBASE,
        LogMessageFlags.NOTIFICATION,
        (PARAM_ITEMS.ID, PARAM_ITEMS.PIN),
    )

    POLIBASE_PERSON_WAS_UPDATED = EventDescription(
        "Обновлена полибейс персона: {name} ({pin})",
        LogMessageChannels.POLIBASE,
        LogMessageFlags.SILENCE,
        (PARAM_ITEMS.NAME, PARAM_ITEMS.PIN, PARAM_ITEMS.VALUE),
    )

    POLIBASE_PERSON_VISIT_NOTIFICATION_WAS_REGISTERED = EventDescription(
        "Зарегистрировано новое уведомление о посещении: {name} ({type_string})",
        LogMessageChannels.POLIBASE,
        LogMessageFlags.SILENCE,
        (
            PARAM_ITEMS.NAME,
            ParamItem("type_string", ""),
            ParamItem("notification", ""),
        ),
    )

    POLIBASE_PERSON_REVIEW_NOTIFICATION_WAS_REGISTERED = EventDescription(
        None,
        LogMessageChannels.POLIBASE,
        (LogMessageFlags.SAVE_ONCE, LogMessageFlags.SILENCE),
        (PARAM_ITEMS.PERSON_PIN.configurate(key=True), PARAM_ITEMS.INPATIENT),
    )

    POLIBASE_PERSON_REVIEW_NOTIFICATION_WAS_ANSWERED = EventDescription(
        "Клиент: {}, ответил на уведомление об отзыве: \"{}\" ({})",
        LogMessageChannels.POLIBASE,
        LogMessageFlags.SAVE,
        (
            PARAM_ITEMS.PERSON_PIN.configurate(key=True),
            PARAM_ITEMS.VALUE,
            PARAM_ITEMS.STATUS,
        ),
    )

    POLIBASE_PERSONS_WITH_OLD_FORMAT_BARCODE_WAS_DETECTED = EventDescription(
        "Полибейс: обнаружены пациенты со старым форматом или отсутствующим штрих-кодом",
        LogMessageChannels.POLIBASE,
        LogMessageFlags.SILENCE,
        (PARAM_ITEMS.PERSON_PIN,),
    )

    POLIBASE_PERSONS_BONUS_CARD_WAS_CREATED = EventDescription(
        "Бонусная карта для клиента {} была создана",
        LogMessageChannels.POLIBASE,
        (LogMessageFlags.NOTIFICATION, LogMessageFlags.SAVE),
        (
            PARAM_ITEMS.PERSON_PIN.configurate(key=True),
            ParamItem("url", "").configurate(visible=False),
        ),
    )

    POLIBASE_PERSON_BARCODES_WITH_OLD_FORMAT_WERE_CREATED = EventDescription(
        "Все штрих-коды для новых клиентов созданы",
        LogMessageChannels.POLIBASE,
        LogMessageFlags.SILENCE,
        (PARAM_ITEMS.PERSON_PIN,),
    )

    POLIBASE_PERSON_WITH_INACCESSABLE_EMAIL_WAS_DETECTED = EventDescription(
        "Клиент {} ({}) имеет недоступную электронную почту: {}. Регистратор: {}, компьютер: {} ({})",
        LogMessageChannels.POLIBASE_ERROR,
        LogMessageFlags.SAVE,
        (
            PARAM_ITEMS.PERSON_NAME,
            PARAM_ITEMS.PERSON_PIN,
            PARAM_ITEMS.EMAIL,
            PARAM_ITEMS.REGISTRATOR_PERSON_NAME,
            ParamItem(FIELD_NAME_COLLECTION.WORKSTATION_NAME, ""),
            ParamItem(FIELD_NAME_COLLECTION.WORKSTATION_DESCRIPTION, ""),
        ),
    )

    POLIBASE_PERSON_EMAIL_WAS_ADDED = EventDescription(
        "Электронная почта клиента {} ({}): {} была добавлена",
        LogMessageChannels.POLIBASE,
        (LogMessageFlags.RESULT, LogMessageFlags.SAVE),
        (
            PARAM_ITEMS.PERSON_NAME,
            PARAM_ITEMS.PERSON_PIN,
            PARAM_ITEMS.EMAIL,
        ),
    )

    ASK_FOR_POLIBASE_PERSON_EMAIL = EventDescription(
        "Запрос электронной почта у клиента {} ({}). Локально: {}",
        LogMessageChannels.POLIBASE,
        (LogMessageFlags.NOTIFICATION, LogMessageFlags.SEND_ONCE),
        (
            PARAM_ITEMS.PERSON_NAME.configurate(saved=False),
            PARAM_ITEMS.PERSON_PIN.configurate(key=True),
            PARAM_ITEMS.LOCALY,
            PARAM_ITEMS.SECRET.configurate(visible=False),
        ),
    )

    POLIBASE_PERSON_BONUSES_WAS_UPDATED = EventDescription(
        "Бонусы клиента {} обновились",
        LogMessageChannels.POLIBASE,
        (LogMessageFlags.NOTIFICATION, LogMessageFlags.SILENCE),
        (PARAM_ITEMS.PERSON_PIN,),
    )

    ACTION_WAS_DONE = EventDescription(
        'Совершено действие "{}"➜{}.\nДействие совершил: {} ({}).\nПараметры действия: {}.\nПринудительное действие: {}.',
        LogMessageChannels.IT_BOT,
        LogMessageFlags.SAVE,
        (
            ParamItem(FIELD_NAME_COLLECTION.ACTION_DESCRIPTION, ""),
            ParamItem(FIELD_NAME_COLLECTION.ACTION_NAME, ""),
            PARAM_ITEMS.NAME,
            PARAM_ITEMS.LOGIN,
            PARAM_ITEMS.PARAMETERS,
            ParamItem(FIELD_NAME_COLLECTION.FORCED, ""),
        ),
    )

    ACTION_HAVE_TO_BE_DONE = EventDescription(
        "Необходимо совершить действие {}: {}",
        LogMessageChannels.DEBUG,
        LogMessageFlags.SAVE,
        (
            ParamItem(FIELD_NAME_COLLECTION.ACTION_NAME, ""),
            ParamItem(FIELD_NAME_COLLECTION.ACTION_DESCRIPTION, ""),
        ),
    )

    NEW_EMAIL_MESSAGE_WAS_RECEIVED = EventDescription(
        "Почтовый ящик: {mailbox}\nНовое письмо было получено от {from}: {title}.",
        LogMessageChannels.NEW_EMAIL,
        LogMessageFlags.NOTIFICATION,
        (
            ParamItem("mailbox", ""),
            PARAM_ITEMS.TITLE,
            ParamItem("from", ""),
            PARAM_ITEMS.VALUE,
        ),
    )

    CARD_REGISTRY_FOLDER_WAS_SET_FOR_POLIBASE_PERSON = EventDescription(
        'Карта пациента: {} добавлена в папку "{}".\nДействие совершил: {} ({})',
        LogMessageChannels.CARD_REGISTRY,
        LogMessageFlags.SAVE_ONCE,
        (
            PARAM_ITEMS.PERSON_PIN.configurate(key=True),
            PARAM_ITEMS.CARD_REGISTRY_FOLDER,
            PARAM_ITEMS.REGISTRATOR_PERSON_NAME,
            PARAM_ITEMS.REGISTRATOR_PERSON_PIN,
        ),
    )

    CARD_REGISTRY_FOLDER_WAS_SET_NOT_FROM_POLIBASE_FOR_POLIBASE_PERSON = (
        EventDescription(
            "Карта пациента: {} добавлена не с помощью Полибейс",
            LogMessageChannels.CARD_REGISTRY,
            LogMessageFlags.SAVE_ONCE,
            (PARAM_ITEMS.PERSON_PIN,),
        )
    )

    CARD_REGISTRY_SUITABLE_FOLDER_WAS_SET_FOR_POLIBASE_PERSON = EventDescription(
        'Карта пациента: {} добавлена в подходящую папку "{}".\nДействие совершил: {} ({})',
        LogMessageChannels.CARD_REGISTRY,
        LogMessageFlags.NOTIFICATION,
        (
            PARAM_ITEMS.PERSON_PIN,
            PARAM_ITEMS.CARD_REGISTRY_FOLDER,
            PARAM_ITEMS.REGISTRATOR_PERSON_NAME,
            PARAM_ITEMS.REGISTRATOR_PERSON_PIN,
        ),
    )

    CARD_REGISTRY_FOLDER_WAS_REGISTERED = EventDescription(
        "Папка с картами пациентов: {} добавлена в реестр карт пациентов. Положение: шкаф: {}; полка: {}; место на полке: {}",
        LogMessageChannels.CARD_REGISTRY,
        LogMessageFlags.SAVE_ONCE,
        (
            PARAM_ITEMS.CARD_REGISTRY_FOLDER.configurate(key=True),
            ParamItem("p_a", ""),
            ParamItem("p_b", ""),
            ParamItem("p_c", ""),
        ),
    )

    CARD_REGISTRY_FOLDER_START_CARD_SORTING = EventDescription(
        'Начат процесс сортировки карта пациентов в папке реестра карт "{}"',
        LogMessageChannels.CARD_REGISTRY,
        LogMessageFlags.SAVE,
        (PARAM_ITEMS.CARD_REGISTRY_FOLDER,),
    )

    CARD_REGISTRY_FOLDER_COMPLETE_CARD_SORTING = EventDescription(
        'Закончен процесс сортировки карта пациентов в папке реестра карт "{}"',
        LogMessageChannels.CARD_REGISTRY,
        LogMessageFlags.SAVE,
        (PARAM_ITEMS.CARD_REGISTRY_FOLDER,),
    )

    EMPLOYEE_CHECKED_IN = EventDescription(
        "Сотрудник {} отметился на приход",
        LogMessageChannels.TIME_TRACKING,
        LogMessageFlags.NOTIFICATION,
        (PARAM_ITEMS.NAME,),
    )

    EMPLOYEE_CHECKED_OUT = EventDescription(
        "Сотрудник {} отметился на выход",
        LogMessageChannels.TIME_TRACKING,
        LogMessageFlags.NOTIFICATION,
        (PARAM_ITEMS.NAME,),
    )

    RESULT_WAS_RETURNED = EventDescription(
        "Возврат результата из мобильного помощника",
        LogMessageChannels.IT_BOT,
        LogMessageFlags.SILENCE,
        (
            ParamItem("key", "Ключ"),
            PARAM_ITEMS.VALUE,
            ParamItem("interaption_type", "Занчение прерывания", optional=True),
        ),
    )

    SAVE_FILE_FROM_KNOWLEDGE_BASE = EventDescription(
        "Сохранение файла из базы знаний (Мобильные файл)",
        LogMessageChannels.IT_BOT,
        LogMessageFlags.SILENCE,
        (
            PARAM_ITEMS.ID.configurate(key=True),
            PARAM_ITEMS.TITLE,
            PARAM_ITEMS.TEXT,
        ),
    )

    SAVE_NOTE_FROM_KNOWLEDGE_BASE = EventDescription(
        "Сохранение заметки из базы знаний (Мобильные заметки)",
        LogMessageChannels.IT_BOT,
        LogMessageFlags.SILENCE,
        (
            PARAM_ITEMS.ID.configurate(key=True),
            PARAM_ITEMS.TITLE,
            PARAM_ITEMS.TEXT,
            PARAM_ITEMS.IMAGES,
        ),
    )



class DATE_TIME:
    SPLITTER: str = "T"
    HOUR_TIME_FORMAT: str = "%H"
    SECONDLESS_TIME_FORMAT: str = f"{HOUR_TIME_FORMAT}:%M"
    SECOND_ZEROS_TIME_FORMAT: str = f"{SECONDLESS_TIME_FORMAT}:00"
    TIME_FORMAT: str = f"{SECONDLESS_TIME_FORMAT}:%S"
    DAY_FORMAT: str = "%d"

    ISO_DATE_FORMAT: str = f"%Y-%m-{DAY_FORMAT}"
    ISO_DATETIME_FORMAT: str = f"{ISO_DATE_FORMAT}{SPLITTER}{SECOND_ZEROS_TIME_FORMAT}"

    DATE_PART_DELIMITER: str = "."
    YEARLESS_DATE_FORMAT: str = f"{DAY_FORMAT}{DATE_PART_DELIMITER}%m"
    DATE_FORMAT: str = f"{YEARLESS_DATE_FORMAT}{DATE_PART_DELIMITER}%Y"
    DAYLESS_DATE_FORMAT: str = f"%m{DATE_PART_DELIMITER}%Y"

    DATETIME_ONLY_HOUR_FORMAT: str = f"{DATE_FORMAT} {HOUR_TIME_FORMAT}"
    DATETIME_SECOND_ZEROS_FORMAT: str = f"{DATE_FORMAT} {SECOND_ZEROS_TIME_FORMAT}"
    DATETIME_FORMAT: str = f"{DATE_FORMAT} {TIME_FORMAT}"



from ipih import FACADE_FOLDER_NAME

from pih.tools import j

class FACADE:
    NAME: str = FACADE_FOLDER_NAME
    SERVICE_FOLDER_SUFFIX: str = "Service"
    
    @staticmethod
    def SERVICE_NAME(service_name: str) -> str:
        suffix: str = FACADE.SERVICE_FOLDER_SUFFIX
        return (
            j(
                (
                    service_name,
                    (None if service_name.endswith(suffix) else suffix),
                )
            )
        )



from pih.consts.file import FILE

class PYTHON:
        EXECUTOR_ALIAS: str = "py"
        EXECUTOR: str = "python"
        EXECUTOR3: str = "python3"
        PIP: str = "pip"
        SEARCH_PATTERN: str = "\\Python\\Python"
        PACKAGE_EXTENSION: str = FILE.EXTENSION.WHEEL

        class COMMAND:
            VERSION: str = "--version"
            INSTALL: str = "install"
            UNINSTALL: str = "uninstall"
            SHOW: str = "show"
            FLAG: str = "-c"


from dataclasses import dataclass

from typing import Any

import grpc

RPCError = grpc.RpcError

class NotImplemented(BaseException):
    pass

class OperationExit(BaseException):
    pass


class OperationCanceled(BaseException):
    pass

@dataclass
class Redirection(BaseException):
    arg: Any | None = None



class ZeroReached(BaseException):
    pass

class BarcodeNotFound(BaseException):

    def get_details(self) -> str:
        return "Штрих-код не распознан, попробуйте еще раз"


class NotFound(BaseException):

    def get_details(self) -> str:
        return self.args[0]

    def get_value(self) -> str:
        return self.args[1]


class IncorrectInputFile(BaseException):
    pass


class NotAccesable(BaseException):
    pass

@dataclass
class Error(BaseException):
    details: str | None = None
    code: tuple | None = None


from pih.collections import PasswordSettings

class RULES:
        SPECIAL_CHARACTER: str = "s"
        LOWERCASE_ALPHABET: str = "a"
        UPPERCASE_ALPHABET: str = "A"
        DIGIT: str = "d"
        DEFAULT_ORDER_LIST: list[str] = [
            SPECIAL_CHARACTER,
            LOWERCASE_ALPHABET,
            UPPERCASE_ALPHABET,
            DIGIT,
        ]

class PASSWORD:
    
    class SETTINGS:
        SIMPLE: PasswordSettings = PasswordSettings(
            3, "", RULES.DEFAULT_ORDER_LIST, 0, 3, 0, 0, False
        )
        NORMAL: PasswordSettings = PasswordSettings(
            8, "!@#", RULES.DEFAULT_ORDER_LIST, 3, 3, 1, 1, False
        )
        STRONG: PasswordSettings = PasswordSettings(
            10,
            "#%+\-!=@()_",
            RULES.DEFAULT_ORDER_LIST,
            3,
            3,
            2,
            2,
            True,
        )
        DEFAULT: PasswordSettings = NORMAL
        PC: PasswordSettings = NORMAL
        EMAIL: PasswordSettings = NORMAL

    def get(name: str) -> SETTINGS:
        return PASSWORD.__getattribute__(PASSWORD.SETTINGS, name)




from tkinter import Text
from typing import Callable

from pih.const import CONST

class BarcodeInput(Text):

    def __init__(self, *args, **kwargs):
        Text.__init__(self, *args, **kwargs)
        self.result_handler: Callable[[str, bool], None] | None = None
        self._orig = self._w + "_orig"
        self.tk.call("rename", self._w, self._orig)
        self.tk.createcommand(self._w, self._proxy)
        self.internal_set: bool = False

        def onModification(event) -> None:
            if not self.internal_set:
                content = event.widget.get("1.0", "end-1c")
                next: int = content.find(BARCODE_READER.PREFIX)
                if next > 0:
                    content = content[next:]
                if len(content) > 0:
                    if content[0] == BARCODE_READER.PREFIX and content[-1] == BARCODE_READER.SUFFIX:
                        self.result_handler(
                            content[len(BARCODE_READER.PREFIX):-len(BARCODE_READER.SUFFIX)])
                    elif content[0] != BARCODE_READER.PREFIX:
                        self.center()
                        self.result_handler(content, False)
                else:
                    self.result_handler(content, False)

        def onEnter(event) -> None:
            content = event.widget.get("1.0", "end-1c")
            self.result_handler(content, True)
            self.set_text("")

        self.bind("<<TextModified>>", onModification)
        self.bind('<Return>', onEnter)


    def set_result_handler(self, value: Callable) -> None:
        self.result_handler = value

    def _proxy(self, command, *args):
        cmd = (self._orig, command) + args
        result = self.tk.call(cmd)
        if command in ("insert", "delete", "replace"):
            self.event_generate("<<TextModified>>")
        return result

    def set_text(self, value: str) -> None:
        self.internal_set = True
        self.delete(1.0, "end")
        self.insert(1.0, value)
        self.center()
        self.internal_set = False

    def center(self) -> None:
        self.tag_configure("tag_name", justify='center')
        self.tag_add("tag_name", "1.0", "end")



from pih.tools import j, jp
from pih.consts.names import USER_PROPERTIES

from enum import Enum, auto

class AD:
    
    ADMINISTRATOR_LOGIN: str = "Administrator"
    SPLITTER: str = "."
    OU: str = "OU="
    SEARCH_ATTRIBUTES: list[str] = [USER_PROPERTIES.LOGIN, USER_PROPERTIES.NAME]
    SEARCH_ATTRIBUTE_DEFAULT: str = SEARCH_ATTRIBUTES[0]
    USER_HOME_FOLDER_DISK: str = "U:"
    SEARCH_ALL_PATTERN: str = "*"
    LOCATION_JOINER: str = ":"
    TEMPLATED_USER_SERACH_TEMPLATE: str = j(("_", SEARCH_ALL_PATTERN, "_"))
    

    USER_ACCOUNT_CONTROL: list[str] = [
        "SCRIPT",
        "ACCOUNTDISABLE",
        "RESERVED",
        "HOMEDIR_REQUIRED",
        "LOCKOUT",
        "PASSWD_NOTREQD",
        "PASSWD_CANT_CHANGE",
        "ENCRYPTED_TEXT_PWD_ALLOWED",
        "TEMP_DUPLICATE_ACCOUNT",
        "NORMAL_ACCOUNT",
        "RESERVED",
        "INTERDOMAIN_TRUST_ACCOUNT",
        "WORKSTATION_TRUST_ACCOUNT",
        "SERVER_TRUST_ACCOUNT",
        "RESERVED",
        "RESERVED",
        "DONT_EXPIRE_PASSWORD",
        "MNS_LOGON_ACCOUNT",
        "SMARTCARD_REQUIRED",
        "TRUSTED_FOR_DELEGATION",
        "NOT_DELEGATED",
        "USE_DES_KEY_ONLY",
        "DONT_REQ_PREAUTH",
        "PASSWORD_EXPIRED",
        "TRUSTED_TO_AUTH_FOR_DELEGATION",
        "RESERVED",
        "PARTIAL_SECRETS_ACCOUNT",
    ]
    
    DOMAIN_NAME: str = "fmv"
    DOMAIN_ALIAS: str = "pih"
    DOMAIN_SUFFIX: str = "lan"
    DOMAIN_DNS: str = jp((DOMAIN_NAME, DOMAIN_SUFFIX))
    DOMAIN_MAIN: str = DOMAIN_DNS
    PATH_ROOT: str = j(("//", DOMAIN_MAIN))

    ROOT_CONTAINER_DN: str = f"{OU}Unit,DC={DOMAIN_NAME},DC={DOMAIN_SUFFIX}"
    WORKSTATIONS_CONTAINER_DN: str = f"{OU}Workstations,{ROOT_CONTAINER_DN}"
    SERVERS_CONTAINER_DN: str = f"{OU}Servers,{ROOT_CONTAINER_DN}"
    USERS_CONTAINER_DN_SUFFIX: str = f"Users,{ROOT_CONTAINER_DN}"
    ACTIVE_USERS_CONTAINER_DN: str = f"{OU}{USERS_CONTAINER_DN_SUFFIX}"
    INACTIVE_USERS_CONTAINER_DN: str = f"{OU}dead{USERS_CONTAINER_DN_SUFFIX}"
    GROUP_CONTAINER_DN: str = f"{OU}Groups,{ROOT_CONTAINER_DN}"
    PROPERTY_ROOT_DN: str = f"{OU}Property,{GROUP_CONTAINER_DN}"
    PROPERTY_COMPUTER_DN: str = f"{OU}Computer,{PROPERTY_ROOT_DN}"
    PROPERTY_USER_DN: str = f"{OU}User,{PROPERTY_ROOT_DN}"
    JOB_POSITION_CONTAINER_DN: str = f"{OU}Job positions,{GROUP_CONTAINER_DN}"

    WORKSTATION_PREFIX_LIST: list[str] = ["ws-", "nb-", "fmvulianna"]

    class USER:
        MARKETER_ADMINISTRATOR: str = "marketer_admin"
        CALL_CENTRE_ADMINISTRATOR: str = "callCentreAdmin"
        REGISTRATION_AND_CALL: str = "reg_and_call"
        CONTROL_SERVICE: str = "cctv"
        INDICATIONS_ALL: str = "indications_all"
        ADMINISTRATOR: str = "Administrator"

    class JobPositions(Enum):
        HR = auto()
        IT = auto()
        CALL_CENTRE = auto()
        REGISTRATOR = auto()
        RD = auto()
        MARKETER = auto()

    class Groups(Enum):
        TimeTrackingReport = auto()
        Inventory = auto()
        Polibase = auto()
        Admin = auto()
        ServiceAdmin = auto()
        CardRegistry = auto()
        PolibaseUsers = auto()
        RD = auto()
        IndicationWatcher = auto()
        FunctionalDiagnostics = auto()

    class ComputerProperties(Enum):
        Watchable = 1
        Shutdownable = 2
        Rebootable = 4
        DiskReportable = 8
        DiskReportableViaZabbix = 16
        
    class UserProperies(Enum):
        Jokeless = 1
        DoctorVisitless = 2
        HasLunchBreak = 4
        TimeTrackingless = 8